/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.edit;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.data.NotificationTypes;
import net.sf.genomeview.gui.menu.AbstractModelAction;
import net.sf.jannot.Feature;
/**
 * 
 * @author Thomas Abeel
 *
 */
public class RemoveAction extends AbstractModelAction implements Observer {

	private static final long serialVersionUID = -3409728329439144492L;

	public RemoveAction(Model model) {
		super("Remove selected feature", model);
		model.addObserver(this);
		setEnabled(model.selectionModel().getFeatureSelection() != null&&model.selectionModel().getFeatureSelection().size()>0);
	}

	public void actionPerformed(ActionEvent arg0) {

		Set<Feature> toRemove = new HashSet<Feature>();
		toRemove.addAll(model.selectionModel().getFeatureSelection());
		for (Feature rf : toRemove)
			model.vlm.getSelectedEntry().getMemoryAnnotation(rf.type()).remove(rf);
		model.refresh(NotificationTypes.JANNOTCHANGE);
	}

	public void update(Observable o, Object arg) {
		setEnabled(model.selectionModel().getFeatureSelection() != null&&model.selectionModel().getFeatureSelection().size()>0);

	}

}
