/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.help;

import java.awt.event.ActionEvent;
import java.util.Collection;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.plugin.PluginLoader;

import org.java.plugin.registry.PluginDescriptor;


public class ShowInstalledModulesAction extends AbstractAction {

    private Model model;

    private static final long serialVersionUID = 4182067300462615334L;

    public ShowInstalledModulesAction(Model model) {
        super("List of installed modules");
        this.model = model;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Collection<PluginDescriptor> col = PluginLoader.pluginManager.getRegistry().getPluginDescriptors();
        String message = "<html>";
        for (PluginDescriptor p : col) {
            // PluginLoader.pluginManager.
            if (p.getId().equals("genomeview.core")) {
                message += "Plugin manager (" + p.getVersion() + ")<br/>";
            } else {
                message += p.getExtensions().iterator().next().getParameter("name").rawValue();
                message += " (" + p.getVersion() + ")<br/>";
                if (p.getDocumentation() != null)
                    message += "&nbsp;&nbsp;" + p.getDocumentation();
            }
            if (p.getExtensions().size() > 0) {
                System.out.println(p.getExtensions().iterator().next().getParameter("name").rawValue());
            }

        }
        message += "</html>";

        JOptionPane.showMessageDialog(model.getGUIManager().getParent(), message, "Installed modules", JOptionPane.INFORMATION_MESSAGE);
    }

}
