/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.selection;

import java.awt.event.ActionEvent;
import java.util.Observable;

import javax.swing.KeyStroke;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.dialog.SequenceViewDialog;
import net.sf.genomeview.gui.menu.AbstractModelAction;


/**
 * Action to display a dialog with nucleotide/protein sequence of the selected
 * CDS.
 * 
 * 
 */
public class ShowSequenceWindowAction extends AbstractModelAction {

    private static final long serialVersionUID = 4601582100774522419L;

    public ShowSequenceWindowAction(Model model) {
        super("Show DNA or protein sequence", model);
        super.putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke("control Q"));
    }

    public void actionPerformed(ActionEvent arg0) {
        assert ((model.selectionModel().getFeatureSelection() != null  && model.selectionModel().getFeatureSelection().size() > 0) || 
        		(model.getSelectedRegion() != null));
        SequenceViewDialog dialog = new SequenceViewDialog(model);
        dialog.showSequenceViewDialog();

    }

    @Override
    public void update(Observable o, Object obj) {
        setEnabled((model.selectionModel().getFeatureSelection() != null && model.selectionModel().getFeatureSelection().size() > 0) ||
        		(model.getSelectedRegion() != null));
    }
}
