/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.viztracks;

import java.awt.Color;
import java.awt.Graphics2D;

import javax.swing.JViewport;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.Convert;
import net.sf.jannot.Location;
import net.sf.jannot.StringKey;

public class TickmarkTrack extends Track {
	public static final StringKey key = new StringKey("GV::TICKMARK");

	public TickmarkTrack(Model model) {
		super(key, model, true, false);
	}
	@Override
	protected void paintDisplayName(Graphics2D g, int yOffset){
		//Do nothing
	}
	
	@Override
	protected int cogOffset(){
		return 14;
	}
	
	@Override
	public int paintTrack(Graphics2D g, int yOffset, double screenWidth,JViewport view,TrackCommunicationModel tcm) {
		Location r = model.vlm.getAnnotationLocationVisible();
		g.setColor(Color.BLACK);
		g.drawLine(0, yOffset + 15, (int) screenWidth + 1, yOffset + 15);

		if (r.start() == r.end()) {
			return 32;
		}
		// determine the tickDistance, we aim for 10 ticks on screen.
		int length = r.length();
		int scale = (int) Math.log10(length / 10.0);
		int multiplier = (int) (length / Math.pow(10, scale + 1));
		int tickDistance = (int) (Math.pow(10, scale) * multiplier);
		if (tickDistance == 0)
			tickDistance = 1;
		// paint the ticks
		int currentTick = (r.start() - r.start() % tickDistance) + 1;
		boolean up = true;
		while (currentTick < r.end()) {
			int xpos = Convert.translateGenomeToScreen(currentTick, r, screenWidth);
			String s = "" + currentTick;

			if (up) {
				g.drawLine(xpos, yOffset + 2, xpos, yOffset + 28);
				g.drawString(s, xpos + 2, yOffset + 14);
			} else {
				g.drawLine(xpos, yOffset + 2, xpos, yOffset + 28);
				g.drawString(s, xpos + 2, yOffset + 26);
			}
			up = !up;

			currentTick += tickDistance;

		}
		return 32;
	}


}
